;------------------------------------------------------------------------------
;            
INIT_LG1    PROC

            ; Disable interrupts while we're setting up
            DIS

            ; Reset the visible flag for the MOB
            MVI     0, R0
            ANDI    #11110111111111b, R0
            MVO     R0, 0

            ; Initialize the state
            MVII    #ISR_STATES -1, R0
            MVO     R0, STATE

            ; Setup the border and color-stack colors
            CLRR    R0
            MVO     R0, BORD_COLOR
            MVO     R0, CS0_COLOR
            MVO     R0, CS1_COLOR
            MVO     R0, CS2_COLOR
            MVO     R0, CS3_COLOR
            
            ; Load-up the backtab
            CALL    LOAD_BTAB
            DECLE   CARDS_LG1

            ; Load the first frame's pixel data
            MVII    #FRAME_SEQ_LG.start, R5
            MVO     R5, NEXT_FRAME
            MVI@    R5, R4
            MVII    #MEMMAP.gram, R1
            MVII    #64, R0
            CALL    COPY_CARDS

            ; Re-enable interrupts and we're done        
            EIS
            
            PULR    PC
            ENDP

;------------------------------------------------------------------------------
;
;
INIT_LG2    PROC

            DIS
            
            ; Load-up the backtab
            CALL    LOAD_BTAB
            DECLE   CARDS_LG2
            
            EIS
            
            PULR    PC
            ENDP

;------------------------------------------------------------------------------
;
;
MODE_LG     PROC
            
            ; Update the state and save it back out
            MVI     STATE, R0
            INCR    R0
            CMPI    #ISR_STATES, R0
            BNEQ    @@save_state
            CLRR    R0
@@save_state
            MVO     R0, STATE

            ;---
            ; Jump to where we should be based on the current state
            TSTR    R0
            BEQ     @@move_p1
            DECR    R0
            BEQ     @@move_p2
            DECR    R0
            BEQ     @@move_p3
            
            ;-----------------------------------------------------------------
            ; Stuff we do when we're not writing pixel data
            ;
            MVI     NEXT_FRAME, R0          ; Check the next frame in the 
            CMPI    #FRAME_SEQ_LG.end, R0      ; sequence and see if we're done
            BNEQ    @@exit_early            ; Only restart if we're at the end

            ; Restart the animation from the first frame
            MVII    #FRAME_SEQ_LG.start, R0  
            MVO     R0, NEXT_FRAME
            
@@exit_early:
            PULR    PC
                        

@@move_p1
            MVI     NEXT_FRAME, R5
            MVI@    R5, R4
            MVO     R5, NEXT_FRAME     
            MVII    #MEMMAP.gram, R1
            MVII    #21, R0
            B       @@copy
            
@@move_p2
            MVI     F_PTR, R4
            MVII    #MEMMAP.gram+(21*8), R1
            MVII    #21, R0
            B       @@copy
            
@@move_p3
            MVI     F_PTR, R4
            MVII    #MEMMAP.gram+(42*8), R1
            MVII    #22, R0
            B       @@copy

@@copy
            CALL    COPY_CARDS
            MVO     R4, F_PTR
                        
@@exit
            PULR    PC
            ENDP

