
;------------------------------------------------------------------------------
;            
INIT_SM1    PROC

            DIS

            ; Setup stuff for the horizontal scrolling            
            MVII    #1, R0
            MVO     R0, STIC.edgemask
            MVII    #H_SCR_UPD_TICKS, R0    ; We'll scroll by a pixel every N  
            MVO     R0, H_SCR_COUNTER       ; ticks as given in H_SCR_UPD_TICKS       
            MVII    #0, R0                  ; No horizontal delay to start with
            MVO     R0, H_DELAY            

            ; Reset the visible flag for the MOB
            MVI     0, R0
            ANDI    #11110111111111b, R0
            MVO     R0, 0
            
            ; Initialize the state
            MVII    #ISR_STATES -1, R0
            MVO     R0, STATE

            ; Setup the border and color-stack colors
            CLRR    R0
            MVO     R0, BORD_COLOR
            MVO     R0, CS0_COLOR
            MVO     R0, CS2_COLOR
            MVO     R0, CS1_COLOR
            MVO     R0, CS3_COLOR
            
            ; Put cards into the BACKTAB
            CALL    LOAD_BTAB
            DECLE   CARDS_SM1

            ; Initialize the pointer into the frame sequence table
            MVII    #FRAME_SEQ_SM.start, R0
            MVO     R0, NEXT_FRAME

            EIS

            PULR    PC
            ENDP            
                        
;------------------------------------------------------------------------------
;            
INIT_SM2    PROC

            DIS

            ; Reset stuff we did for the horizontal scrolling
            MVII    #0, R0
            MVO     R0, STIC.edgemask
            CLRR    R0
            MVO     R0, STIC.h_delay 
            MVO     R0, H_DELAY
            
            ; Setup the color-stack 
            MVII    #2, R0
            MVO     R0, CS0_COLOR
            MVII    #5, R0
            MVO     R0, CS1_COLOR
            MVII    #1, R0
            MVO     R0, CS2_COLOR
            MVII    #10, R0
            MVO     R0, CS3_COLOR

            ; Initialize the pointer into the MOB sequence table
            MVII    #MOB_SEQ.start, R0
            MVO     R0, NEXT_MOB

            ; Initialize the Running Man on MOB 0
            MVII    #8 OR STIC.mobx_visb, R0
            MVO     R0, MOB_SHAD.x0

            MVII    #52 OR STIC.moby_yres, R0
            MVO     R0, MOB_SHAD.y0

            MVII    #C_WHT OR (30*8) OR STIC.moba_gram, R0
            MVO     R0, MOB_SHAD.a0

            ; Update the MOB on the next clock tick
            MVII    #1, R0              
            MVO     R0, MOB_UPD_COUNTER

            ; Set velocities
            MVII    #MOB_X_VEL_INIT, R0
            MVO     R0, MOB_X_VEL
            MVII    #MOB_Y_VEL_INIT, R0
            MVO     R0, MOB_Y_VEL

            ; Put cards into the BACKTAB
            CALL    LOAD_BTAB
            DECLE   CARDS_SM2

            EIS
                                    
            PULR    PC
            ENDP            

;------------------------------------------------------------------------------
;
;
MODE_SM1    PROC

            MVI     H_SCR_COUNTER, R0
            DECR    R0
            BNEQ    @@save_h_scr_counter
            CALL    H_SCROLL
            MVII    #H_SCR_UPD_TICKS, R0
            
@@save_h_scr_counter
            MVO     R0, H_SCR_COUNTER            

            MVII    #@@ret, R5
            PSHR    R5
            MVII    #MODE_SM2.common_entry, R7
@@ret
            PULR    PC
            ENDP
            
;------------------------------------------------------------------------------
;
;
MODE_SM2    PROC
            
            ; Copy MOB shadow
            MVII    #MOB_SHAD, R4
            MVII    #STIC, R5
            MVII    #MOB_SHAD.end - MOB_SHAD, R1            
@@shad_copy
            MVI@    R4, R0
            MVO@    R0, R5
            DECR    R1
            BNEQ    @@shad_copy
            
@@common_entry            
            ; Update the state and save it back out
            MVI     STATE, R0
            INCR    R0
            CMPI    #ISR_STATES, R0
            BNEQ    @@save_state
            CLRR    R0
@@save_state
            MVO     R0, STATE

            ;---
            ; Jump to where we should be based on the current state
            TSTR    R0
            BEQ     @@move_p1
            DECR    R0
            BEQ     @@move_p2
            B       @@every_tick

@@move_p1
            MVI     NEXT_FRAME, R5
            MVI@    R5, R4
            MVO     R5, NEXT_FRAME    
            MVII    #MEMMAP.gram, R1
            MVII    #15, R0
            CALL    COPY_CARDS
            MVO     R4, F_PTR
            B       @@every_tick

@@move_p2
            MVI     F_PTR, R4
            MVII    #MEMMAP.gram+(15*8), R1
            MVII    #15, R0         ; Second half of baby + MOB data
            CALL    COPY_CARDS
            MVO     R4, F_PTR

@@every_tick
            ; Keep the MOB animation up to date
            MVI     MOB_UPD_COUNTER, R0
            DECR    R0
            BNEQ    @@save_mob_counter
           
            ; Copy next MOB animation frame into the GRAM            
            MVI     NEXT_MOB, R5
            MVI@    R5, R4
            MVO     R5, NEXT_MOB
            MVII    #MEMMAP.gram+(30*8), R1
            MVII    #2, R0
            CALL    COPY_CARDS     
            
            CALL    MOVE_MOB
            MVII    #MOB_UPD_TICKS, R0

@@save_mob_counter
            MVO     R0, MOB_UPD_COUNTER  

            ; Bounds check on the frame sequence
            MVI     NEXT_FRAME, R0          ; Check the next frame in the
            CMPI    #FRAME_SEQ_SM.end, R0      ; sequence and see if we're done
            BNEQ    @@check_mob             ; Only restart if we're at the end

            ; Restart the animation from the first frame
            MVII    #FRAME_SEQ_SM.start, R0
            MVO     R0, NEXT_FRAME

@@check_mob
            MVI     NEXT_MOB, R0
            CMPI    #MOB_SEQ.end, R0
            BNEQ    @@exit

            MVII    #MOB_SEQ.start, R0
            MVO     R0, NEXT_MOB

@@exit
            PULR    PC
            ENDP
                                   
;------------------------------------------------------------------------------   
;
MOVE_MOB    PROC
            BEGIN

            ;-----------------------------
            ; Check X
            MVI     MOB_SHAD.x0, R1
            MOVR    R1, R0
            MVI     MOB_X_VEL, R2
            ANDI    #$FF00, R1      ; High word in R1
            ANDI    #$00FF, R0      ; Low word in R0

            ADDR    R2, R0          ; Add the x velocity
            CMPI    #160, R0
            BGE     @@reverse_x
            CMPI    #8, R0
            BGT     @@write_x

@@reverse_x
            NEGR    R2              ; Reverse direction
            ADDR    R2, R0
            MVO     R2, MOB_X_VEL
            MVI     MOB_SHAD.y0, R2
            XORI    #STIC.moby_xflip, R2
            MVO     R2, MOB_SHAD.y0

@@write_x
            ADDR    R0, R1
            MVO     R1, MOB_SHAD.x0

            ;-------------------------
            ; Check Y
            MVI     MOB_SHAD.y0, R1
            MOVR    R1, R0
            MVI     MOB_Y_VEL, R2

            ANDI    #$FF80, R1
            ANDI    #$007F, R0

            ADDR    R2, R0
            CMPI    #96, R0
            BGE     @@reverse_y
            CMPI    #8, R0
            BGT     @@write_y

@@reverse_y
            NEGR    R2
            ADDR    R2, R0
            MVO     R2, MOB_Y_VEL

@@write_y
            ADDR    R0, R1
            MVO     R1, MOB_SHAD.y0

            RETURN
            ENDP

;------------------------------------------------------------------------------
;            
H_SCROLL    PROC
            BEGIN
            
            ; Test horiz scroll
            MVI     H_DELAY, R0
            INCR    R0
            CMPI    #8, R0
            BNEQ    @@save_scroll
            CLRR    R0
            
@@save_scroll            
            MVO     R0, STIC.h_delay
            MVO     R0, H_DELAY
            TSTR    R0
            BNZE    @@ret
            
            ; Move the backtab cards around to scroll right by one card
            MVII    #$200,  R4
            MVII    #$201,  R5
            MOVR    R4,     R3
            MVII    #12,    R2

@@do_row
            MVI@    R4,     R0      ; rd  0 at  0

            MVI@    R5,     R1      ; rd  1 at  1
            MVO@    R0,     R4      ; wr  0 at  1
            MVI@    R4,     R0      ; rd  2 at  2
            MVO@    R1,     R5      ; wr  1 at  2

            MVI@    R5,     R1      ; rd  3 at  3
            MVO@    R0,     R4      ; wr  2 at  3
            MVI@    R4,     R0      ; rd  4 at  4
            MVO@    R1,     R5      ; wr  3 at  4

            MVI@    R5,     R1      ; rd  5 at  5
            MVO@    R0,     R4      ; wr  4 at  5
            MVI@    R4,     R0      ; rd  6 at  6
            MVO@    R1,     R5      ; wr  5 at  6

            MVI@    R5,     R1      ; rd  7 at  7
            MVO@    R0,     R4      ; wr  6 at  7
            MVI@    R4,     R0      ; rd  8 at  8
            MVO@    R1,     R5      ; wr  7 at  8

            MVI@    R5,     R1      ; rd  9 at  9
            MVO@    R0,     R4      ; wr  8 at  9
            MVI@    R4,     R0      ; rd 10 at 10
            MVO@    R1,     R5      ; wr  9 at 10

            MVI@    R5,     R1      ; rd 11 at 11
            MVO@    R0,     R4      ; wr 10 at 11
            MVI@    R4,     R0      ; rd 12 at 12
            MVO@    R1,     R5      ; wr 11 at 12

            MVI@    R5,     R1      ; rd 13 at 13
            MVO@    R0,     R4      ; wr 12 at 13
            MVI@    R4,     R0      ; rd 14 at 14
            MVO@    R1,     R5      ; wr 13 at 14

            MVI@    R5,     R1      ; rd 15 at 15
            MVO@    R0,     R4      ; wr 14 at 15
            MVI@    R4,     R0      ; rd 16 at 16
            MVO@    R1,     R5      ; wr 15 at 16

            MVI@    R5,     R1      ; rd 17 at 17
            MVO@    R0,     R4      ; wr 16 at 17
            MVI@    R4,     R0      ; rd 18 at 18
            MVO@    R1,     R5      ; wr 17 at 18

            MVI@    R5,     R1      ; rd 19 at 19
            MVO@    R0,     R4      ; wr 18 at 19

            MVO@    R1,     R3      ; wr 19 at  0

            INCR    R5

            ADDI    #20,    R3
            
            DECR    R2
            BNEQ    @@do_row
            
@@ret
            RETURN
            ENDP
            